#! /usr/local/bin/wish -f

if [info exists fredtcl_loaded] {
    return;
}

set	fredtcl_loaded	1

set	fredtcl_help(help)	\
"To browse, move to the main window and double-click on an entry.\n\nIf you double-click with the left button, then any subordinates to that entry are listed; instead, if you double-click with the right button, then a new window containing information about that entry will appear.\n\n\nTo search, click on \"Find...\".\n\nEnter the name, organization, and locality/country of the person you're interested in, e.g.,\n\n\trose, dbc, us\n\nor\n\n\tpiskor, trm, ottawa, on, ca"


wm title \
	.		$myname
wm iconbitmap \
	.		@$fred_library/fredtcl.icon

frame	.title		-borderwidth 2 \
			-relief raised
pack append \
	.title		[button .title.find \
				-command CallbackFind \
				-text Find...] \
			    {left} \
			[label .title.msg \
				-text ""] \
			    {left expand} \
			[button .title.quit \
				-command CallbackQuit \
				-text Quit] \
			    {right} \
			[button .title.help \
				-command "CallbackHelp .help \"Help with $myname\" help help" \
				-text Help] \
			    {right} \
			[button .title.options \
				-command CallbackOptions \
				-text Options] \
			    {right} \
			[button .title.addmds \
				-command CallbackADDMDs \
				-text Providers] \
			    {right}

frame	.frame		-borderwidth 10

pack append \
	.frame		[scrollbar .frame.yscroll \
				-command ".frame.list yview" \
				-orient vertical \
				-relief sunken] \
			    {right filly} \
			[scrollbar .frame.xscroll \
				-command ".frame.list xview" \
				-orient horizontal \
				-relief sunken] \
			    {bottom fillx} \
			[listbox .frame.list \
				-font -*-courier-medium-r-normal-*-120-* \
				-geometry 70x30 \
				-relief sunken \
				-xscroll ".frame.xscroll set" \
				-yscroll ".frame.yscroll set"] \
			    {top expand}
# -font -*-*-medium-r-normal-*-*-0-*-m-*-iso8859-*

bind	.frame.list	<Double-1>	ExpandCommand
bind	.frame.list	<3>		{%W select from [%W nearest %y]}
bind	.frame.list	<B3-Motion>	{%W select to [%W nearest %y]}
bind	.frame.list	<Shift-3>	{%W select adjust [%W nearest %y]}
bind	.frame.list	<Shift-B3-Motion> \
					{%W select to [%W nearest %y]}
bind	.frame.list	<Double-3>	ShowCommand

catch {	.frame.list	configure	-background lightskyblue }

pack append \
	.		.title \
			    {top pady 5 fill} \
			.frame \
			    {bottom fill}


#
# Callback: Find something...
#

set	fredtcl_name	""

proc	CallbackFind	{{w .find}} {
    global	myname fredtcl_name fred_library

    mkMsg	""

    catch	{destroy $w}
    toplevel	$w

    wm title	$w		"Find someone using $myname"
    wm iconname	$w		"find"
    wm iconbitmap \
		$w		@$fred_library/fredtcl.icon

    frame	$w.frame	-borderwidth 10
    pack append	$w.frame	[label $w.frame.lb \
					-text "Find: "] \
				    {left} \
				[entry $w.frame.en \
					-borderwidth 2 \
					-font -*-courier-medium-r-normal-*-120-* \
					-relief sunken \
					-textvariable fredtcl_name \
					-width 60] \
				    {left}

    bind	$w.frame.en	<Return>	"$w.bot.find flash; $w.bot.find configure -state active; $w.bot.find invoke"
    bind	$w		<Any-Enter>	"focus $w.frame.en"

    frame	$w.bot		-borderwidth 1
    pack append	$w.bot		[button	$w.bot.find \
					-command "FindCommandAux; destroy $w" \
					-text Search] \
				    {left expand padx 20 pady 5} \
				[button	$w.bot.cancel \
					-command "destroy $w" \
					-text Cancel] \
				    {left expand padx 20 pady 5}

    pack append	$w		$w.frame \
				    {top expand filly} \
				$w.bot \
				    {bottom fill}

}

proc	FindCommandAux	{} {
    global	approx phone wild fredtcl_name
    global	fredtcl_ptr0 fredtcl_ptr1 fredtcl_ptr2 fredtcl_ptr3

    set		fredtcl_ptr0	""
    set		fredtcl_ptr1	""
    set		fredtcl_ptr2	""
    set		fredtcl_ptr3	""

    set		command		"dish fred -ufn -audio,-jpeg,-list,"
    if {$phone == 1} {
	append	command		"-phone,"
    }
    append	command		"-photo,-ptr,"
    set		options		0
    if {$approx == 1} {
	incr	options		1
    }
    if {$wild == 1} {
	incr	options		2
    }
    append	command		[format "-options %x," $options]
    append	command		"$fredtcl_name"

    eval $command
}


#
# Callback: Expand an entry in the name tree
#

set	fredtcl_names	""

proc	ExpandCommand {} {
    global	exhaustive fullistings fredtcl_names
    global	fredtcl_ptr0 fredtcl_ptr1 fredtcl_ptr2 fredtcl_ptr3

    mkMsg	""

    set		fredtcl_ptr0	""
    set		fredtcl_ptr1	""
    set		fredtcl_ptr2	""
    set		fredtcl_ptr3	""

    foreach	i	[.frame.list curselection] {
	set	dn	[lindex $fredtcl_names $i]
	set	command	"dish fred -expand "
	if {$fullistings == 1} {
	    append command	"-full "
	}
	if {$exhaustive == 1} {
	    append command	"-exhaustive "
	}
	append	command	"\"@$dn\""

	eval	$command
    }
}


#
# Callback: Show an entry in the name tree
#

proc	ShowCommand {} {
    global	fredtcl_names

    foreach	i	[.frame.list curselection] {
	set	dn	[lindex $fredtcl_names $i]
	ShowCommandAux	"$dn"
    }
}

proc	ShowCommandAux	{dn} {
    global	phone
    global	fredtcl_ptr0 fredtcl_ptr1 fredtcl_ptr2 fredtcl_ptr3

    mkMsg	""

    set		fredtcl_ptr0	""
    set		fredtcl_ptr1	""
    set		fredtcl_ptr2	""
    set		fredtcl_ptr3	""

    set		command		"dish showentry -fred -fredaudio -fredjpeg -fredlist -fredphoto -fredptr -nofredseq -dontdereferencealias "
    if {$phone == 1} {
	append	command		"-fredphone "
    }
    append	command	"\"@$dn\""

    eval	$command
}


#
# Callback: Help with fredtcl
#

proc	CallbackHelp		{w title icon msg} {
    global	myname fred_library fredtcl_help

    mkMsg	""

    catch	{destroy $w}
    toplevel	$w

    wm title	$w		"$title"
    wm iconname	$w		"$icon"
    wm iconbitmap \
		$w		@$fred_library/fredtcl.icon

    frame	$w.frame	-borderwidth 10
    pack append	$w.frame	[message $w.frame.msg \
					-font -*-courier-medium-r-normal-*-180-* \
					-text "$fredtcl_help($msg)"] \
				    {expand}

    frame	$w.bot		-borderwidth 1
    pack append	$w.bot		[button	$w.bot.button \
					-command "destroy $w" \
					-text Dismiss] \
				    {top padx 5 pady 5 expand}

    pack append	$w		$w.frame \
				    {top expand filly} \
				$w.bot \
				    {bottom fill}
}


#
# Callback: Options using fredtcl
#	someday allow configuration of audio/jpeg/photo and DSA/DA-server
#

set	fredtcl_help(options)	\
"The \"Application\" option determines, when an entry is presented, whether telephony or e-mail information is presented on the first line.\n\nThe \"Listings\" options determine, when browsing, whether all subordinates (complete) or only those subordinates corresponding to organizations and people (useful) are listed; and, whether $myname should be aggressive in looking for subordinates.\n\nThe \"Matching\" options determine, when searching, whether phonetic matching (approximate) and/or substring matching (wildcard) should be used.\n\nFinally, the \"Miscellany\" options are used for debugging purposes."

proc	CallbackOptions		{{w .options}} {
    global	myname fred_library

    mkMsg	""

    catch	{destroy $w}
    toplevel	$w

    wm title	$w		"Options using $myname"
    wm iconname	$w		"options"

    wm iconbitmap \
		$w		@$fred_library/fredtcl.icon

    frame	$w.frame	-borderwidth 10

    pack append $w.frame	[frame $w.frame.application -borderwidth 5] \
				    {top pady 4 expand frame w} \
				[frame $w.frame.listings -borderwidth 5] \
				    {top pady 4 expand frame w} \
				[frame $w.frame.matching -borderwidth 5] \
				    {top pady 4 expand frame w} \
				[frame $w.frame.misc -borderwidth 5] \
				    {top pady 4 expand frame w}

    pack append	$w.frame.application \
				[label $w.frame.application.lb \
					-text Application \
					-width 12] \
				    {left filly} \
				[radiobutton $w.frame.application.rb0 \
					-command "CallbackBoolean phone 1" \
					-relief flat \
					-text "Telephony  " \
					-value 1 \
					-variable phone] \
				    {left padx 4 expand} \
				[radiobutton $w.frame.application.rb1 \
					-command "CallbackBoolean phone 0" \
					-relief flat \
					-text "E-mail" \
					-value 0 \
					-variable phone] \
				    {left padx 4 expand}

    pack append	$w.frame.listings \
				[label $w.frame.listings.lb \
					-text Listings \
					-width 12] \
				    {left filly} \
				[radiobutton $w.frame.listings.rb0 \
					-command "CallbackBoolean fullistings 1" \
					-relief flat \
					-text "Complete    " \
					-value 1 \
					-variable fullistings] \
				    {left padx 4 expand} \
				[radiobutton $w.frame.listings.rb1 \
					-command "CallbackBoolean fullistings 0" \
					-relief flat \
					-text "Useful        " \
					-value 0 \
					-variable fullistings] \
				    {left padx 4 expand} \
				[checkbutton $w.frame.listings.exhaustive \
					-command "CallbackBoolean exhaustive" \
					-relief flat \
					-text "Exhaustive" \
					-variable exhaustive] \
				    {left padx 4 expand}

    pack append	$w.frame.matching \
				[label $w.frame.matching.lb \
					-text Matching \
					-width 12] \
				    {left filly} \
				[checkbutton $w.frame.matching.approx \
					-command "CallbackBoolean approx" \
					-relief flat \
					-text "Approximate" \
					-variable approx] \
				    {left padx 4 expand} \
				[checkbutton $w.frame.matching.wild \
					-command "CallbackBoolean wild" \
					-relief flat \
					-text "Wildcard" \
					-variable wild] \
				    {left padx 4 expand}

    pack append	$w.frame.misc \
				[label $w.frame.misc.lb \
					-text Miscellany \
					-width 12] \
				    {left filly} \
				[checkbutton $w.frame.misc.verbose \
					-command "CallbackBoolean verbose" \
					-relief flat \
					-text "Verbose       " \
					-variable verbose] \
				    {left padx 4 expand} \
				[checkbutton $w.frame.misc.watch \
					-command "CallbackBoolean watch" \
					-relief flat \
					-text "Watch         " \
					-variable watch] \
				    {left padx 4 expand} \
				[checkbutton $w.frame.misc.debug \
					-command "CallbackBoolean debug" \
					-relief flat \
					-text Debug \
					-variable debug] \
				    {left padx 4 expand}

    frame	$w.bot		-borderwidth 1
    pack append	$w.bot		[button	$w.bot.help \
					-command "CallbackHelp .helpopt \"Help with $myname options\" \"option help\" options" \
					-text Help] \
				    {left expand padx 20 pady 5}
    pack append	$w.bot		[button	$w.bot.dismiss \
					-command "destroy $w" \
					-text Dismiss] \
				    {left expand padx 20 pady 5}

    pack append	$w		$w.frame \
				    {top expand filly} \
				$w.bot \
				    {bottom fill}
}


#
# Callback: Providers
#

set	fredtcl_addmds	""
set	fredtcl_addmdx(-1)	\
			""
set	fredtcl_addmdy(-1)	\
			""
set	fredtcl_addmdz(-1)	\
			""

set	fredtcl_addmdid	0
set	fredtcl_mkaddmd	0
set	fredtcl_help(addmds)	\
"When searching, one or more Public Directory Service Providers (PDSPs) may hold additional information.  If, after selecting any or all of the PDSPs from the list below, you click on \"Apply\", then your choices will be automatically consulted, as appropriate.\n\nIf PDSPs other than any of your selections hold information, then you will be asked for confirmation before the search continues."


proc	CallbackADDMDs		{{w .addmds}} {
    global	exhaustive fredtcl_addmds fredtcl_addmdy fredtcl_addmdz
    global	fredtcl_addmdid fredtcl_mkaddmd fredtcl_names fred_library

    mkMsg	""

    if {$fredtcl_addmdid == 0} {
	set	fredtcl_mkaddmd	1

	for {set i 0; set j [llength $fredtcl_names]} {$i<$j} {incr i} {
	    set	dn	[lindex $fredtcl_names $i]
	    if {[string first "@"  "$dn"] >= 0}	{ continue }
	    if {[string first "c=" "$dn"] != 0}	{ continue }

	    set	command	"dish fred -expand -addmd "
	    if {$exhaustive == 1} {
	        append command	"-exhaustive "
	    }
	    append	command	"\"@$dn\""

	    eval	$command
	}
	mkMsg	""

	set	fredtcl_mkaddmd	0

	set	fredtcl_addmdid	1
    }

    if {[llength $fredtcl_addmds] == 0} {
	bell 100
	mkMsg	"no Providers found..."
	return
    }

    catch	{destroy $w}
    toplevel	$w

    wm title	$w		"Providers"
    wm iconname	$w		"addmds"

    wm iconbitmap \
		$w		@$fred_library/fredtcl.icon

    frame	$w.frame	-borderwidth 10

    for {set i 0; set j [llength $fredtcl_addmds]} {$i<$j} {incr i} {
	pack append $w.frame	[checkbutton $w.frame.$i \
					-relief flat \
					-text "$fredtcl_addmdz($i)" \
					-variable fredtcl_addmdy($i)] \
				    {top pady 4 expand frame w}
    }

    frame	$w.bot		-borderwidth 1
    pack append	$w.bot		[button	$w.bot.apply \
					-command "applyADDMDs; destroy $w" \
					-text Apply] \
				    {left expand padx 20 pady 5}
    pack append	$w.bot		[button	$w.bot.reset \
					-command "resetADDMDs" \
					-text Reset] \
				    {left expand padx 20 pady 5}
    pack append	$w.bot		[button	$w.bot.help \
					-command "CallbackHelp .helpad \"Help with Providers\" \"addmd help\" addmds" \
					-text Help] \
				    {left expand padx 20 pady 5}
    pack append	$w.bot		[button	$w.bot.dismiss \
					-command "resetADDMDs; destroy $w" \
					-text Dismiss] \
				    {left expand padx 20 pady 5}

    pack append	$w		$w.frame \
				    {top expand filly} \
				$w.bot \
				    {bottom fill}
}

#
# Here start the internal routines...
#

proc	mkDialog		{w msgArgs args} {
    global	myname fred_library

    catch	{destroy $w}
    toplevel	$w		-class Dialog

    wm title	$w		"$myname dialog"
    wm iconbitmap \
		$w		@$fred_library/fredtcl.icon

    frame	$w.frame	-borderwidth 1
    eval message \
		$w.frame.msg	-font -*-times-medium-r-normal-*-180* \
				-justify center \
				$msgArgs
    pack append	$w.frame	$w.frame.msg \
				    {top expand padx 5 pady 5}
    catch { $w.frame.msg configure -background palevioletred }

    if {[llength $args] > 0} {
	frame		$w.bot	-borderwidth 1
	pack append	$w.bot	[frame $w.bot.0 \
					-borderwidth 1 \
					-relief sunken] \
				    {left expand padx 20 pady 20}

	set		arg	[lindex $args 0]
	pack append	$w.bot.0 \
				[button $w.bot.0.button \
					-command "[lindex $arg 1];destroy $w" \
					-text [lindex $arg 0]] \
				    {expand padx 12 pady 12}
	bind		$w	<Return>	"[lindex $arg 1]; destroy $w"
	focus		$w

	set		i	1
	foreach		arg	[lrange $args 1 end] {
	    pack append	$w.bot	[button	$w.bot.$i \
					-command "[lindex $arg 1];destroy $w" \
					-text [lindex $arg 0]] \
				    {left expand padx 20}
	    incr	i
	}
    } else {
	frame		$w.bot	-borderwidth 1
	pack append	$w.bot	[button	$w.bot.button \
					-command "destroy $w" \
					-text Dismiss] \
				    {top padx 5 pady 5 expand}
    }

    pack append		$w	$w.frame \
				    {top fill expand} \
				$w.bot \
				    {top fill expand}


    bind		$w	<Any-Enter>	[list focus $w]
    focus		$w

    wm geometry		$w	+300+300
    bind		$w	<Visibility>	"grab $w; focus $w"

    tkwait window	$w
}


set	fredtcl_armed	0
set	fredtcl_ignore	0

proc	mkMsg		{msg} {
    global	fredtcl_armed fredtcl_ignore

    if {$fredtcl_ignore == 1} {
	set	fredtcl_ignore	0
	return
    }
    incr	fredtcl_armed
    if {$msg == ""} {
	catch { .title.msg configure -background bisque1 }
	.title.msg configure	-text ""
    } else {
	catch { .title.msg configure -background palevioletred }
	.title.msg configure	-text "$msg"
	after	30000		"mkMsgAux $fredtcl_armed"
    }
}

proc	mkMsgAux	{serial} {
    global	fredtcl_armed

    if {$serial == $fredtcl_armed} {
	mkMsg	""
    }
}

set	fredtcl_ptr0	""
set	fredtcl_ptr1	""
set	fredtcl_ptr2	""
set	fredtcl_ptr3	""

proc	mkText		{code title} {
    global	fredtcl_buffer fredtcl_ignore fredtcl_names fredtcl_mkaddmd
    global	fred_library
    global	fredtcl_ptr0 fredtcl_ptr1 fredtcl_ptr2 fredtcl_ptr3

    if {$fredtcl_mkaddmd} {
	return
    }
    if {$code == 4} {
	set	line		[string range $fredtcl_buffer 0 \
			          [expr [string first "\n" $fredtcl_buffer]-1]]
	set	entry		[split "$line" "$"]
	set	title		[fred name head [set ufn [lindex $entry 0]]]
	set	dn		[lindex $entry 1]
	mkListAux		"$dn"	"$ufn"
	set	offset		[fred search $fredtcl_names $dn]
	set	w		.info$offset
    } else {
	set	w		.info
	set	title		"Information"
	set	msg		[split "$fredtcl_buffer" "\n"]
	if {[llength $msg] == 2 && [lindex $msg 1] == ""} {
	    catch	{destroy $w}

	    mkMsg	[lindex $msg 0]
	    set		fredtcl_ignore	1
	    return
	}
    }

    catch	{destroy $w}
    toplevel	$w
    addBusy	$w

    wm title	$w		"$title"
    wm iconbitmap \
		$w		@$fred_library/fredtcl.icon

    if {[set pl [llength $fredtcl_ptr0]] > 0} {
	frame	$w.top		-borderwidth 2 \
				-relief raised

	set	menus	""
	set	mlen	0
	for {set i 0} {$i < $pl} {incr i} {
	    if {[lindex $fredtcl_ptr3 $i] != ""} {
		pack append	$w.top	[button	$w.top.$i \
					    -command "exec [lindex $fredtcl_ptr3 $i]" \
					    -text [lindex $fredtcl_ptr0 $i]] \
					    {left}

		continue
	    }
	    if {[set j [lsearch $menus [set attr [lindex $fredtcl_ptr0 $i]]]] \
		     == -1} {
		lappend		menus	$attr
		set		j	$mlen
		incr		mlen

		pack append	$w.top	[menubutton $w.top.mb$j \
						-menu $w.top.mb$j.menu \
						-relief raised \
						-text $attr] \
					    {left}
		menu $w.top.mb$j.menu
	    }

	    set		dn		[lindex $fredtcl_ptr2 $i]
	    $w.top.mb$j.menu add	command -command "ShowCommandAux \"$dn\"" \
						-label [lindex $fredtcl_ptr1 $i]
	}
	
	pack append \
		$w		$w.top \
				    {top fillx}

	set		fredtcl_ptr0	""
	set		fredtcl_ptr1	""
	set		fredtcl_ptr2	""
	set		fredtcl_ptr3	""
    }

    frame	$w.frame	-borderwidth 10
    pack append	$w.frame	[scrollbar $w.frame.yscroll \
					-command "$w.frame.list yview" \
					-orient vertical \
					-relief sunken] \
				    {right filly} \
				[scrollbar $w.frame.xscroll \
					-command "$w.frame.list xview" \
					-orient horizontal \
					-relief sunken] \
				    {bottom fillx} \
				[listbox $w.frame.list \
					-font -*-courier-medium-r-normal-*-120-* \
					-geometry 80x20 \
					-relief sunken \
					-xscroll "$w.frame.xscroll set" \
					-yscroll "$w.frame.yscroll set"] \
				    {top expand}
    foreach	i		[split "$fredtcl_buffer" "\n"] {
	if {$code == 4} {
	    set	code	1
	} else {
	    $w.frame.list	insert	end $i
	}
    }
    catch { $w.frame.list configure -background lightgoldenrod }

    frame	$w.bot		-borderwidth 1
    pack append	$w.bot		[button	$w.bot.button \
					-command "delBusy $w; destroy $w" \
					-text Dismiss] \
				    {top padx 5 pady 5 expand}

    pack append	$w		$w.frame \
				    {top expand filly} \
				$w.bot \
				    {bottom fill}
}


proc	mkPtrs		{} {
    global	fredtcl_buffer
    global	fredtcl_ptr0 fredtcl_ptr1 fredtcl_ptr2 fredtcl_ptr3

    if {[llength [set entry [split "$fredtcl_buffer" "$"]]] == 3} {
	mkListAux	[lindex $entry 2] [lindex $entry 1]

	lappend		fredtcl_ptr0	[lindex $entry 0]
	lappend		fredtcl_ptr1	[lindex $entry 1]
	lappend		fredtcl_ptr2	[lindex $entry 2]
	lappend		fredtcl_ptr3	{}
    }
}


proc	mkVals		{} {
    global	fredtcl_buffer
    global	fredtcl_ptr0 fredtcl_ptr1 fredtcl_ptr2 fredtcl_ptr3

    if {[llength [set entry [split "$fredtcl_buffer" "$"]]] == 2} {
	lappend		fredtcl_ptr0	[lindex $entry 0]
	lappend		fredtcl_ptr1	{}
	lappend		fredtcl_ptr2	{}
	lappend		fredtcl_ptr3	[lindex $entry 1]
    }
}


proc	mkList		{} {
    global	fredtcl_buffer fredtcl_mkaddmd

    foreach	i		[split "$fredtcl_buffer" "\n"] {
	if {[llength [set entry [split "$i" "$"]]] == 2} {
	    if {$fredtcl_mkaddmd} {
		mkADDMD		[lindex $entry 1] [lindex $entry 0]
	    } else {
		mkListAux	[lindex $entry 1] [lindex $entry 0]
	    }
	}
    }
}

proc	mkADDMD		{dn ufn} {
    global	fredtcl_addmds fredtcl_addmdx fredtcl_addmdy fredtcl_addmdz

    if {[fred search $fredtcl_addmds $dn] != -1} {
	return
    }

    set		i			[llength $fredtcl_addmds]
    lappend	fredtcl_addmds		"$dn"
    set		fredtcl_addmdx($i)	0
    set		fredtcl_addmdy($i)	$fredtcl_addmdx($i)
    set		fredtcl_addmdz($i)	"$ufn"
}

proc	mkListAux	{dn ufn} {
    global	fredtcl_names

# puts stdout " dn=$dn"
# puts stdout "ufn=$ufn"
    if {[fred search $fredtcl_names $dn] != -1} {
	return
    }

    if {[set di [string last  "@" "$dn"]] > 0} {
	mkListAux	[string range "$dn" 0 [expr $di-1]] \
			[fred name tail "$ufn"]

	set	dfn		"  "
	for {set i [expr $di-1]} {$i>=0} {incr i -1} {
	    if {[string index "$dn" $i] == "@"} {
		append	dfn	"  "
	    }
	}
	append	dfn		[fred name head "$ufn"]
    } else {
	set	dfn		$ufn
    }

    lappend	fredtcl_names	"$dn"
    set		fredtcl_names	[fred sort $fredtcl_names]
    .frame.list	insert	[fred search $fredtcl_names $dn] "$dfn"
}


proc	mkListing	{title} {
    global	fredtcl_buffer fred_library

    foreach	i		[split "$fredtcl_buffer" "\n"] {
	set	entry		[split "$i" "$"]
	mkListAux		[lindex $entry 1] [lindex $entry 0]
    }

    set		w		.list

    catch	{destroy $w}
    toplevel	$w

    wm title	$w		"$title"
    wm iconname	$w		"choices"
    wm iconbitmap \
		$w		@$fred_library/fredtcl.icon

    frame	$w.frame	-borderwidth 10
    pack append	$w.frame	[scrollbar $w.frame.yscroll \
					-command "$w.frame.list yview" \
					-orient vertical \
					-relief sunken] \
				    {right expand filly frame w} \
				[listbox $w.frame.list \
					-font -*-courier-medium-r-normal-*-120-* \
					-geometry 80x15 \
					-relief sunken \
					-yscroll "$w.frame.yscroll set"] \
				    {left expand filly frame e}

    foreach	i		[split "$fredtcl_buffer" "\n"] {
	set	entry		[split "$i" "$"]
	$w.frame.list	insert	end [lindex $entry 0]
    }
    catch { $w.frame.list configure -background khaki }

    frame	$w.bot		-borderwidth 1
    pack append	$w.bot		[button	$w.bot.button \
					-command "mkListingAux $w.frame.list; destroy $w" \
					-text Dismiss] \
				    {top padx 5 pady 5 expand}

    pack append	$w		$w.frame \
				    {top expand filly} \
				$w.bot \
				    {bottom fill}

    tkwait window	$w
    update
}

proc	mkListingAux	{w} {
    global	fredtcl_buffer

    set		fredtcl_buffer	""
    foreach	i	[$w curselection] {
	append	fredtcl_buffer	" "
	append	fredtcl_buffer	[expr $i+1]
    }
}


proc	applyADDMDs {}	{
    global	fredtcl_addmds fredtcl_addmdx fredtcl_addmdy

    set	command	"dish fred -ufnad"
    set	dnsep	" \""
    set	didone	0
    for {set i 0; set j [llength $fredtcl_addmds]} {$i<$j} {incr i} {
	if {[set fredtcl_addmdx($i) $fredtcl_addmdy($i)]} {
	    append	command "$dnsep"
	    append	command	[lindex $fredtcl_addmds $i]
	    set		dnsep	"\\$"
	    set		didone	1
	}
    }
    if {$didone} {
	append	command	"\""
    }

    eval	$command
}

proc	resetADDMDs {}	{
    global	fredtcl_addmds fredtcl_addmdx fredtcl_addmdy

    for {set i 0; set j [llength $fredtcl_addmds]} {$i<$j} {incr i} {
	set	fredtcl_addmdy($i)	$fredtcl_addmdx($i)
    }
}


proc	dumpListing {}	{
    global	fredtcl_names

    for {set i 0; set j [llength $fredtcl_names]} {$i<$j} {incr i} {
	set	dfn	[.frame.list get $i]
	set	dn	[lindex $fredtcl_names $i]
	puts stdout "$dfn\$$dn"
    }
}


set	fredtcl_busy	[list . .find .addmds]

proc	mkBusy		{} {
    global	fredtcl_busy

    for {set i 0; set j [llength $fredtcl_busy]} {$i<$j} {incr i} {
	catch { busy hold [lindex $fredtcl_busy $i] }
    }
}

proc	notBusy		{} {
    global	fredtcl_busy

    for {set i 0; set j [llength $fredtcl_busy]} {$i<$j} {incr i} {
	catch { busy forget [lindex $fredtcl_busy $i] }
    }
}

proc	addBusy		{w} {
    global	fredtcl_busy

    lappend	fredtcl_busy	$w
}

proc	delBusy		{w} {
    global	fredtcl_busy

    if {[set i [lsearch $fredtcl_busy $w]] != -1} {
	set	fredtcl_busy	[lreplace $fredtcl_busy $i $i]
    }
}